# test inputs for 02old.t
$HOST='hadrian'; 
$PROMPT=0;	# don't prompt
$EXISTING_NIC_HDL='PG6-AP'; 
$EXISTING_AS_NUM='AS4777'; 
$EXISTING_MNTNER_NO_AUTH='MAINT-NULL';
$EXISTING_MNTNER_WITH_AUTH='MAINT-APNIC-AP';
$QUERY_THAT_SHOULD_FAIL='SHOULD_FAIL';
1;
